# -*- coding: utf-8 -*-

''' Task to handle communication with driver.

Copyright (C) 2015 CAEN ELS d.o.o.

'''

from PyQt5.QtCore import *
import numpy as np

import DriverInterface as di


class DriverThread(QThread):
    ''' Communciation between Oscilloscope GUI and MPS-Pico8 Linux driver '''

    newDataAvailable = pyqtSignal(np.ndarray)

    def __init__(self, pico_path):
        super(DriverThread, self).__init__()
        self.interval_ms = 1000
        self.exiting = False
        self.nr_samp = 10000
        self.drv = di.DriverInterface(pico_path, debug=False)

    def __del__(self):
        # print('DriverThread __del__')
        self.wait()

    @pyqtSlot()
    def setExiting(self):
        ''' Qt Slot to gracefully terminate thread. '''
        self.exiting = True

    @pyqtSlot(int)
    def set_nr_samp(self, nr_samp):
        self.nr_samp = nr_samp

    @pyqtSlot(str)
    def set_nr_samp(self, nr_samp):
        # print('set_nr_samp', nr_samp)
        self.nr_samp = int(nr_samp, 0)

    @pyqtSlot(float)
    def change_samp_freq(self, samp_freq):
        self.drv.set_fsamp(samp_freq)

    @pyqtSlot(int)
    def change_ranges(self, rngs):
        # print('Changing ranges, captain!')
        # print('Ranges', hex(rngs))
        self.drv.set_range(rngs)

    @pyqtSlot(int)
    def change_gate_mux(self, sel):
        # print('change_gate_mux slot')
        self.drv.set_gate_mux(sel)

    @pyqtSlot(int)
    def change_conv_mux(self, sel):
        # print('change_conv_mux slot')
        self.drv.set_conv_mux(sel)

    @pyqtSlot(int)
    def change_ring_buf(self, nr_samp):
        # print('change_ring_buf slot', nr_samp)
        self.drv.set_ring_buf(nr_samp)

    @pyqtSlot(int, float, int, int)
    def change_trigger_conf(self, mode, level, length, channel):
        # print('change_trigger_conf slot', mode, level, length, channel)
        self.drv.set_trigger(level, channel, length, mode)
    
    # READ THRESHOLD FLAGS
    @pyqtSlot(int)
    def read_threshold_flag(self, data, addr):
        return self.drv.get_threshold_flag(data, addr)

    # READ/WRITE THRESHOLDS 
    @pyqtSlot(float, int)
    def change_threshold(self, data, addr):
        # print('change_ring_buf slot', nr_samp)
        self.drv.set_threshold(data, addr)

    @pyqtSlot(float, int)
    def read_threshold(self, data, addr):
        return self.drv.get_threshold(data, addr)

    # READ/WRITE NSAMP
    @pyqtSlot(int, int)
    def change_nsamp(self, data, addr):
        # print('change_ring_buf slot', nr_samp)
        self.drv.set_nsamp(data, addr)

    @pyqtSlot(int, int)
    def read_nsamp(self, data, addr):
        return self.drv.get_nsamp(data, addr)

    @pyqtSlot(int, int)
    def change_mps_config(self, enable, mps_mux):
        return self.drv.set_mps_config(enable, mps_mux)
    
    @pyqtSlot(int, int)
    def read_mps_config(self, enable, mps_mux):
        return self.drv.get_mps_config(enable, mps_mux)

    @pyqtSlot(int)
    def change_mps_ctrl(self, data):
        return self.drv.set_mps_ctrl(data)

    @pyqtSlot(int, int, int)
    def change_acq_mode(self, mode, acq_samples, delay_samples):
        return self.drv.set_acq_mode(mode, acq_samples, delay_samples)

    @pyqtSlot(int, int)
    def change_trg_acq_delay_samples(self, acq_samples, delay_samples):
        return self.drv.set_trg_acq_samples(acq_samples, delay_samples)

    @pyqtSlot(int, int)
    def read_trg_acq_samples(self, data):
        return self.drv.get_trg_acq_samples(data)
    
    @pyqtSlot(int, int)
    def read_trg_delay_samples(self, data):
        return self.drv.get_trg_delay_samples(data)

    def run(self):
        ''' Infinite loop with sleep() to not overload the processor. '''
        while not self.exiting:
            data = self.drv.read(self.nr_samp).T
            self.newDataAvailable.emit(data)

            if self.nr_samp >= 10000:
                #print('sleeping some more')
                self.msleep(1000)

            if self.nr_samp >= 100000:
                #print('sleeping some more')
                self.msleep(1000)

            self.msleep(self.interval_ms)
            # print('Hello from DriverThread while loop')
