#!/usr/bin/env python3
# -*- coding: utf-8 -*-

''' CAENels MPS-PICO8 Oscilloscope

Copyright (C) 2019 CAEN ELS s.r.l.
'''

import sys
import numpy as np
import time
from PyQt5 import QtGui, uic
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtCore import *
import pyqtgraph as pg
import re
import os

import DriverThread
import TriggerThread
import oscUtils
from BpmGraphs import BpmGraphs
from PicoSelector import PicoSelector
from Config import Config

__version__ = '1.3.2'

N_CHANNELS = int(8)
LABEL_STAT_FIRST_INFINITE = "Mean"
LABEL_STAT_SECOND_INFINITE = "Std dev"
LABEL_STAT_FIRST_TRG = "Mean Buffer 1M"
LABEL_STAT_SECOND_TRG = "Mean Buffer 1k"

pos = np.array([0.0, 0.5, 1.0])
color = np.array([[0, 0, 0, 255], [255, 128, 0, 255], [255, 255, 0, 255]], dtype=np.ubyte)
cmap = pg.ColorMap(pos, color)

pg.setConfigOption('background', (50, 50, 50))
pg.setConfigOption('foreground', (200, 200, 200))
pg.setConfigOption('antialias', True)

class config_class:
    def __init__(self):
        self.th_h_0 = 100e-6
        self.th_h_1 = 100e-6
        self.th_h_2 = 100e-6
        self.th_h_3 = 100e-6
        self.th_h_4 = 100e-6
        self.th_h_5 = 100e-6
        self.th_h_6 = 100e-6
        self.th_h_7 = 100e-6
        self.nsamp_h = int(10)
        self.th_m_0 = 10e-6
        self.th_m_1 = 10e-6
        self.th_m_2 = 10e-6
        self.th_m_3 = 10e-6
        self.th_m_4 = 10e-6
        self.th_m_5 = 10e-6
        self.th_m_6 = 10e-6
        self.th_m_7 = 10e-6
        self.nsamp_m = int(100)
        self.th_l_0 = 1e-6
        self.th_l_1 = 1e-6
        self.th_l_2 = 1e-6
        self.th_l_3 = 1e-6
        self.th_l_4 = 1e-6
        self.th_l_5 = 1e-6
        self.th_l_6 = 1e-6
        self.th_l_7 = 1e-6
        self.nsamp_l = int(100)

def getScriptPath():
    return os.path.dirname(os.path.realpath(sys.argv[0]))


class UserWindow(QtGui.QMainWindow):

    closeThreadSignal = pyqtSignal()
    closeTriggerThreadSignal = pyqtSignal()
    rangesChanged = pyqtSignal(int)
    #gateMuxChanged = pyqtSignal(int)
    #convMuxChanged = pyqtSignal(int)
    ringBufChanged = pyqtSignal(int)
    triggerChanged = pyqtSignal(int, int, int)

    colors = ["#FFFF00", "#FF1493", "#00BFFF", "#00FF7F",
          "#FF8C00", "#BA55D3", "#87CEEB", "#BAFF00"]
    color_lime = "#00FF00"
    color_red  = "#FF0000"

    def __init__(self, pico_path):
        ''' Raises:
            * RuntimeError if the user did not select a pico
        '''

        QtGui.QMainWindow.__init__(self)

        self.ui = uic.loadUi(getScriptPath() + "/mainwindow.ui")
        self.ui.show()

        self.ch_enabled = [1 for i in range(8)]
        self.paused = False

        self.timer = QTimer()


        # Driver thread
        self.driver_thread = DriverThread.DriverThread(pico_path=pico_path)
        self.closeThreadSignal.connect(self.driver_thread.setExiting)
        self.driver_thread.newDataAvailable.connect(self.redrawGraphs)
        self.rangesChanged.connect(self.driver_thread.change_ranges)
        #self.gateMuxChanged.connect(self.driver_thread.change_gate_mux)
        #self.convMuxChanged.connect(self.driver_thread.change_conv_mux)
        self.ringBufChanged.connect(self.driver_thread.change_ring_buf)
        #self.triggerChanged.connect(self.driver_thread.change_trigger_conf)
        self.ui.nrOfSamplesBox.currentIndexChanged[int].connect(self.driver_thread.set_nr_samp)
        self.triggerChanged.connect(self.driver_thread.change_acq_mode)
        self.driver_thread.start()

        # Trigger Thread
        self.nsamp_h_infinite = int(0)
        self.nsamp_l_infinite = int(0)
        self.nsamp_h_trigger = int(0)
        self.nsamp_l_trigger = int(0)
        self.trigger_thread = TriggerThread.TriggerThread(pico_path=pico_path)
        self.closeTriggerThreadSignal.connect(self.trigger_thread.setExiting)
        self.trigger_thread.newTriggerDataAvailable.connect(self.redrawStats)
        self.trigger_thread.start()

        self.ui.statusBar.showMessage('Using: ' + pico_path)

        # BPM graphs
        self.ui.actionMath.triggered.connect(self.on_math_button_released)
        self.bpmGraph = BpmGraphs(self)

        # Configuration Signals
        self.ui.actionConfiguration.triggered.connect(self.on_Configuration_button_released)
        self.config_ui = Config()
        self.config_ui.ui.apply_h.released.connect(self.on_Configuration_apply_h)
        self.config_ui.ui.apply_m.released.connect(self.on_Configuration_apply_m)
        self.config_ui.ui.apply_l.released.connect(self.on_Configuration_apply_l)
        self.config_ui.ui.load_default_1mA.released.connect(self.on_Configuration_load_default_1mA)
        self.config_ui.ui.load_default_1uA.released.connect(self.on_Configuration_load_default_1uA)

        # About Info
        self.ui.actionAbout.triggered.connect(self.on_about_button_released)        
        

        # Mean and stdev lineedits, labels
        self.ui.label_stat_first.setText(LABEL_STAT_FIRST_INFINITE)
        self.ui.label_stat_second.setText(LABEL_STAT_SECOND_INFINITE)
        self.label_ch_stat = \
            [self.ui.findChild(QtGui.QLabel, 'ch' + str(i) + 'Label') for i in range(8)]
        self.lineedit_mean = \
            [self.ui.findChild(QtGui.QLineEdit, 'meanCh' + str(i)) for i in range(8)]
        self.lineedit_stdev = \
            [self.ui.findChild(QtGui.QLineEdit, 'stDevCh' + str(i)) for i in range(8)]
        self.label_ch_rng = \
            [self.ui.findChild(QtGui.QLabel, 'ch' + str(i) + 'RangeLabel') for i in range(8)]
        self.radio_rng1m = \
            [self.ui.findChild(QtGui.QRadioButton, 'rngCh' + str(i) + '_mA') for i in range(8)]
        self.checkbox_enable = \
            [self.ui.findChild(QtGui.QCheckBox, 'ch' + str(i) + 'EnableCheckbox') for i in range(8)]
        self.label_th_h = \
            [self.ui.findChild(QtGui.QLabel, 'label_th_h_' + str(i)) for i in range(8)]
        self.label_th_m = \
            [self.ui.findChild(QtGui.QLabel, 'label_th_m_' + str(i)) for i in range(8)]
        self.label_th_l = \
            [self.ui.findChild(QtGui.QLabel, 'label_th_l_' + str(i)) for i in range(8)]

        # Initialize ranges
        for i in range(8):
            self.radio_rng1m[i].setChecked(True)
        self.on_rangeSelectors_changed()

        # Color
        for i in range(8):
            self.label_ch_stat[i].setStyleSheet('QLabel{ background-color : ' +
                self.colors[i] + ' ; }')
            self.label_ch_rng[i].setStyleSheet('QLabel{ background-color : ' +
                self.colors[i] + ' ; }')

        # Initialize Saturation Flags
        for i in range(8):
            self.label_th_h[i].setStyleSheet('QLabel{ background-color : ' + self.color_lime + ' ; }')
            self.label_th_m[i].setStyleSheet('QLabel{ background-color : ' + self.color_lime + ' ; }')
            self.label_th_l[i].setStyleSheet('QLabel{ background-color : ' + self.color_lime + ' ; }')
        self.th_h_flags = int(0)
        self.th_m_flags = int(0)
        self.th_l_flags = int(0)
        self.update_threshold_flags_on_gui(self.th_h_flags, self.th_m_flags, self.th_l_flags)

        # Initialize plots
        self.ui.currentsGraph.showGrid(x=True, y=True)
        self.ui.currentsGraph_1k.showGrid(x=True, y=True)

        self.ch_plot = []
        self.ch_plot_1k = []

        for i in range(8):
            self.ch_plot.append(self.ui.currentsGraph.plot(pen=self.colors[i]))
            self.ch_plot_1k.append(self.ui.currentsGraph_1k.plot(pen=self.colors[i]))

        # Initialize buttons
        self.ui.pauseButton.setIcon(self.style().standardIcon(QtGui.QStyle.SP_MediaPause))
        self.ui.pauseButton.released.connect(self.on_pauseButton_released)

        # connect range radio buttons to slots
        for i in range(8):
            # Only one of the range button should be connected to slot
            self.radio_rng1m[i].toggled.connect(self.on_rangeSelectors_changed)

        # enable all channels at startup
        for i in range(8):
            self.checkbox_enable[i].stateChanged[int].connect(self.on_enableCheckbox_changed)

        # Trigger widget connections
        self.trg_acq_samples = int(0)
        self.trg_acq_samples = self.driver_thread.read_trg_acq_samples(self.trg_acq_samples)
        self.ui.lineEdit_acq_time.setText(str(self.trg_acq_samples))

        self.trg_delay_samples = int(0)
        self.trg_delay_samples = self.driver_thread.read_trg_delay_samples(self.trg_delay_samples)
        self.ui.lineEdit_acq_time.setText(str(self.trg_delay_samples))
    
        self.ui.comboBox_acq_mode.currentIndexChanged.connect(self.emit_trigger_control)
        self.ui.lineEdit_acq_time.editingFinished.connect(self.on_acq_time_changed)
        self.ui.lineEdit_delay_time.editingFinished.connect(self.on_delay_time_changed)

        # MPS Widget connections
        self.ui.comboBox_MPS.currentIndexChanged.connect(self.on_mlvds_line_changed)
        self.ui.checkBox_enableMPS.stateChanged.connect(self.on_enabledCheckbox_MPS_changed)
        self.ui.pushButton_clearMPS.released.connect(self.on_pushButton_clearMPS)
        self.mps_enable = 0
        self.mps_mux = 0
        self.mps_enable, self.mps_mux = self.driver_thread.read_mps_config(self.mps_enable, self.mps_mux)
        self.ui.comboBox_MPS.setCurrentIndex(int(self.mps_mux))
        self.ui.checkBox_enableMPS.setChecked(int(self.mps_enable))


    def __del__(self):
        # print('mainwindow __del__')
        self.closeThreadSignal.emit()
        self.driver_thread.wait()
        self.closeTriggerThreadSignal.emit()
        self.trigger_thread.wait()

    ########################################################################################
    # UPDATE THRESHOLD FLAGS
    ########################################################################################
    def update_threshold_flags_on_gui(self, th_h_flags, th_m_flags, th_l_flags):
        for i in range(8):
            if ( ((th_h_flags >> i) & 0x1) == 1):
                self.label_th_h[i].setStyleSheet('QLabel{ background-color : ' + self.color_red + ' ; }')
            else:
                self.label_th_h[i].setStyleSheet('QLabel{ background-color : ' + self.color_lime + ' ; }')

            if ( ((th_m_flags >> i) & 0x1) == 1):
                self.label_th_m[i].setStyleSheet('QLabel{ background-color : ' + self.color_red + ' ; }')
            else:
                self.label_th_m[i].setStyleSheet('QLabel{ background-color : ' + self.color_lime + ' ; }')

            if ( ((th_l_flags >> i) & 0x1) == 1):
                self.label_th_l[i].setStyleSheet('QLabel{ background-color : ' + self.color_red + ' ; }')
            else:
                self.label_th_l[i].setStyleSheet('QLabel{ background-color : ' + self.color_lime + ' ; }')

    ########################################################################################
    # UPDATE PLOT MAIN WINDOW
    ########################################################################################
    @pyqtSlot(np.ndarray)
    def redrawGraphs(self, data):
        ''' Redraws plots

        Args:
            data: Numpy array of shape (16, N)
        '''
        if self.paused or len(data[0]) == 0:
            return

        n_samp = len(data[0])
        n_samp_1k = n_samp // 1000
        n_samp_1M = int(n_samp - n_samp_1k)
        
        self.set_trigger_status_triggered()
        self.timer.singleShot(500, self.set_trigger_status_waiting)

        # 1 MHz data
        for i in range(8):
            if self.ch_enabled[i]:
                self.ch_plot[i].setData(oscUtils.wind_avg(data[i,0:n_samp_1M], 1))
                self.lineedit_mean[i].setText(oscUtils.toSIstring(np.mean(data[i,0:n_samp_1M])) + 'A')
                self.lineedit_stdev[i].setText(oscUtils.toSIstring(np.std(oscUtils.wind_avg(data[i,0:n_samp_1M], 1))) + 'A')
            else:
                self.ch_plot[i].setData([0])
                self.lineedit_mean[i].setText('')
                self.lineedit_stdev[i].setText('')

        # 1 kHz data
        for i in range(8):
            if self.ch_enabled[i]:
                self.ch_plot_1k[i].setData(data[i,n_samp_1M:])
            else:
                self.ch_plot_1k[i].setData([0])


        if self.bpmGraph and self.bpmGraph.isVisible():
            self.bpmGraph.draw_plot(data)

        # update saturation flags
        self.th_h_flags = self.driver_thread.read_threshold_flag(self.th_h_flags, 0)
        self.th_m_flags = self.driver_thread.read_threshold_flag(self.th_m_flags, 1)
        self.th_l_flags = self.driver_thread.read_threshold_flag(self.th_l_flags, 2)
        self.update_threshold_flags_on_gui(self.th_h_flags, self.th_m_flags, self.th_l_flags)

    ########################################################################################
    # TRIGGER THREAD REDRAW STATS
    ########################################################################################
    @pyqtSlot(np.ndarray, np.ndarray)
    def redrawStats(self, data, cntr):
        ''' Redraws Stats
        Args:
            data: Numpy array of shape (16, 1) and (2, 1)
        '''
        if self.paused == False:  # if paused is False -> Inifinite mode
            return

        #print("data and cntr: ", data[0], cntr[0], data[8], cntr[1])
        if (cntr[0] > 1000):
            data[0:8] = data[0:8]/1000
        else:
            data[0:8] = data[0:8]/cntr[0]
        if (cntr[1] > 0):
            data[8:16] = data[8:16]/cntr[1]
        for i in range(8):
            if self.ch_enabled[i]:                
                self.ch_plot[i].setData(np.ones((10))*data[i])
                self.lineedit_mean[i].setText(oscUtils.toSIstring(data[i]) + 'A')  # to be divided by cntr
                if (cntr[1] > 0):
                    self.ch_plot_1k[i].setData(np.ones((10))*data[i+8])
                    self.lineedit_stdev[i].setText(oscUtils.toSIstring(data[i+8]) + 'A')
                else:
                    self.ch_plot_1k[i].setData(np.zeros((10)))
                    self.lineedit_stdev[i].setText('0 A')
            else:
                self.ch_plot[i].setData([0])
                self.ch_plot_1k[i].setData([0])
                self.lineedit_mean[i].setText('')
                self.lineedit_stdev[i].setText('')

        self.ui.lineEdit_cntr_first.setText(str(int(cntr[0])))
        self.ui.lineEdit_cntr_second.setText(str(int(cntr[1])))

    ########################################################################################
    # MAIN WINDOW FUNCTIONS AND SIGNALS
    ########################################################################################
    def on_rangeSelectors_changed(self):
        ''' Converts the ranges to MPS-PICO8 range format and changes range '''
        ranges = 0
        for i in range(7, -1, -1):
            ranges <<= 1
            if not self.radio_rng1m[i].isChecked():
                ranges += 1

        self.rangesChanged.emit(ranges)

    def on_pauseButton_released(self):
        ''' Pauses currently displayed screen '''
        self.paused = not self.paused
        if self.paused:
            self.ui.pauseButton.setIcon(self.style().standardIcon(QtGui.QStyle.SP_MediaPlay))
            self.ui.pauseButton.setText('Start')
        else:
            self.ui.pauseButton.setIcon(self.style().standardIcon(QtGui.QStyle.SP_MediaPause))
            self.ui.pauseButton.setText('Pause')

    def on_enableCheckbox_changed(self, val):
        ''' Displays or hides certain channels '''
        chName = self.sender().objectName()
        ch_int = int(re.search(r'\d', chName).group())
        self.ch_enabled[ch_int] = val / 2

    def on_gateMuxBox_changed(self, val):
        # print('on_gateMuxBox_changed', val)
        if val == 2:
            # Show trigger controls
            self.ui.trgCtrlWidget.setEnabled(True)
            # Show cursors
            self.level_cursor.setPen(width=2, color='r')
            self.delay_cursor.setPen(width=2, color='r')
        else:
            # Disable trigger controls
            self.ui.trgCtrlWidget.setEnabled(False)
            # Hide cursors
            self.level_cursor.setPen(None)
            self.delay_cursor.setPen(None)

            self.ringBufChanged.emit(0)

        self.gateMuxChanged.emit(val)

    def on_math_button_released(self):
        # print('on_math_button_released')
        self.bpmGraph.ui.show()

    def on_about_button_released(self):
        # print('on_about_button_released')
        QMessageBox.information(self,"About","v" + str(__version__))


    def emit_trigger_control(self):
        ''' Fetches all trigger controls and passes it to driver layer '''
        if (self.trg_acq_samples < 15):
            self.trg_acq_samples = int(15)
            self.ui.lineEdit_acq_time.setText('15')
        elif (self.trg_acq_samples > 10000):
            self.trg_acq_samples = int(10000)
            self.ui.lineEdit_acq_time.setText('10000')

        if (self.trg_delay_samples < 0):
            self.trg_delay_samples = int(0)
            self.ui.lineEdit_delay_time.setText('0')
        elif (self.trg_delay_samples > 1000000):
            self.trg_delay_samples = int(1000000)
            self.ui.lineEdit_delay_time.setText('1000000')

        mode_idx = self.ui.comboBox_acq_mode.currentIndex()
        if (mode_idx == 0):
            mode = 0   # Infinite Mode
            self.ui.label_stat_first.setText(LABEL_STAT_FIRST_INFINITE)
            self.ui.label_stat_second.setText(LABEL_STAT_SECOND_INFINITE)
            # restore previous nsamp values (if coming back from trigger mode)
            self.driver_thread.change_nsamp(self.nsamp_h_infinite, 0)
            self.driver_thread.change_nsamp(self.nsamp_l_infinite, 2)
            if (self.paused):
                self.on_pauseButton_released()
        else:
            self.on_pauseButton_released()
            mode = 3   # Trigger Mode and MPS signals kept at 0
            self.ui.label_stat_first.setText(LABEL_STAT_FIRST_TRG)
            self.ui.label_stat_second.setText(LABEL_STAT_SECOND_TRG)
            # save nsamp infinite mode
            nsamp = int(0)
            self.nsamp_h_infinite = self.driver_thread.read_nsamp(data=nsamp, addr=0)
            self.nsamp_l_infinite = self.driver_thread.read_nsamp(data=nsamp, addr=2)
            # write new nsamp for trigger mode
            self.nsamp_h_trigger = 1000 if (self.trg_acq_samples > 1000) else self.trg_acq_samples
            self.nsamp_l_trigger = 1 if (self.trg_acq_samples < 1000) else self.trg_acq_samples // 1000
            self.driver_thread.change_nsamp(self.nsamp_h_trigger, 0)
            self.driver_thread.change_nsamp(self.nsamp_l_trigger, 2)

        print("mode:", mode, " acq_samples:", self.trg_acq_samples, " delay_samples:", self.trg_delay_samples)
        # change acquisition mode        
        self.triggerChanged.emit(mode, self.trg_acq_samples, self.trg_delay_samples)

    def on_acq_time_changed(self):
        # this is in unit of us (1 MHz)
        self.trg_acq_samples = int(self.ui.lineEdit_acq_time.text())
        if (self.trg_acq_samples < 15):
            self.trg_acq_samples = int(15)
            self.ui.lineEdit_acq_time.setText('15')
        elif (self.trg_acq_samples > 10000):
            self.trg_acq_samples = int(10000)
            self.ui.lineEdit_acq_time.setText('10000')
        # save nsamp infinite mode
        nsamp = int(0)
        self.nsamp_h_infinite = self.driver_thread.read_nsamp(data=nsamp, addr=0)
        self.nsamp_l_infinite = self.driver_thread.read_nsamp(data=nsamp, addr=2)
        # write new nsamp for trigger mode
        self.nsamp_h_trigger = 1000 if (self.trg_acq_samples > 1000) else self.trg_acq_samples
        self.nsamp_l_trigger = 1 if (self.trg_acq_samples < 1000) else self.trg_acq_samples // 1000
        self.driver_thread.change_nsamp(self.nsamp_h_trigger, 0)
        self.driver_thread.change_nsamp(self.nsamp_l_trigger, 2)
        # change acquisition mode
        self.driver_thread.change_trg_acq_delay_samples(self.trg_acq_samples, self.trg_delay_samples)
        #print("acq time ", self.trg_acq_samples)
    
    def on_delay_time_changed(self):
        # this is in unit of us (1 MHz)
        self.trg_delay_samples = int(self.ui.lineEdit_delay_time.text())
        if (self.trg_delay_samples < 0):
            self.trg_delay_samples = int(0)
            self.ui.lineEdit_delay_time.setText('0')
        elif (self.trg_delay_samples > 1000000):
            self.trg_delay_samples = int(1000000)
            self.ui.lineEdit_delay_time.setText('1000000')
        self.driver_thread.change_trg_acq_delay_samples(self.trg_acq_samples, self.trg_delay_samples)
        #print("delay time ", self.trg_delay_samples)

    def delay_position_changed(self):
        ''' Sets the number of samples before trigger '''
        # print(type(self.sender()))
        if type(self.sender()) == pg.graphicsItems.InfiniteLine.InfiniteLine:
            rb_delay = int(self.sender().getXPos())
        else:
            rb_delay = self.sender().value()

        if rb_delay < 0:
            rb_delay = 0

        if rb_delay > 1023:
            rb_delay = 1023

        self.delay_cursor.setValue(rb_delay)
        self.ui.trgPosSpinBox.setValue(rb_delay)
        self.ringBufChanged.emit(rb_delay)

    def set_trigger_status_waiting(self):
        self.ui.trgStatus.setText('Waiting...')
        self.ui.trgStatus.setStyleSheet('QLabel{ background-color : red ; }')

    def set_trigger_status_triggered(self):
        self.ui.trgStatus.setText('Triggered')
        self.ui.trgStatus.setStyleSheet('QLabel{ background-color : green ; }')

    def on_mlvds_line_changed(self):
        if (self.ui.checkBox_enableMPS.isChecked()):
            self.driver_thread.change_mps_config(1, int(self.ui.comboBox_MPS.currentIndex()))
        else:
            self.driver_thread.change_mps_config(0, int(self.ui.comboBox_MPS.currentIndex()))
    
    def on_enabledCheckbox_MPS_changed(self):
        if (self.ui.checkBox_enableMPS.isChecked()):
            enabled = 1
        else:
            enabled = 0
        self.driver_thread.change_mps_config(enabled, int(self.ui.comboBox_MPS.currentIndex()))

    def on_pushButton_clearMPS(self):
        self.driver_thread.change_mps_ctrl(1)
        time.sleep(1.0)
        self.driver_thread.change_mps_ctrl(0)


    ########################################################################################
    # Configuration Functions
    ########################################################################################
    def on_Configuration_load_default_1mA(self):
        th_array = np.zeros([N_CHANNELS*3,1])
        nsamp_array = np.zeros([3,1])
        coeff = 0.0
        nsamp = int(1)
        for ind_i in range(0,N_CHANNELS*3):
            if ind_i < N_CHANNELS:
                th_array[ind_i,0] = 100e-6
            elif (ind_i >= N_CHANNELS & ind_i < 2*N_CHANNELS):
                th_array[ind_i,0] = 10e-6
            else:
                th_array[ind_i,0] = 1e-6
        nsamp_array[0,0] = 10
        nsamp_array[1,0] = 100
        nsamp_array[2,0] = 100
        self.update_configuration_gui(th_array, nsamp_array)
    
    def on_Configuration_load_default_1uA(self):
        th_array = np.zeros([N_CHANNELS*3,1])
        nsamp_array = np.zeros([3,1])
        coeff = 0.0
        nsamp = int(1)
        for ind_i in range(0,N_CHANNELS*3):
            if ind_i < N_CHANNELS:
                th_array[ind_i,0] = 100e-9
            elif (ind_i >= N_CHANNELS & ind_i < 2*N_CHANNELS):
                th_array[ind_i,0] = 10e-9
            else:
                th_array[ind_i,0] = 1e-9
        nsamp_array[0,0] = 10
        nsamp_array[1,0] = 100
        nsamp_array[2,0] = 100
        self.update_configuration_gui(th_array, nsamp_array)

    def on_Configuration_button_released(self):
        self.config_ui.ui.show()
        self.read_configuration_from_fpga()

    def on_Configuration_apply_h(self):
        config = config_class()
        config = self.config_ui.get_configuration(config)
        self.driver_thread.change_threshold(config.th_h_0, 0)
        self.driver_thread.change_threshold(config.th_h_1, 1)
        self.driver_thread.change_threshold(config.th_h_2, 2)
        self.driver_thread.change_threshold(config.th_h_3, 3)
        self.driver_thread.change_threshold(config.th_h_4, 4)
        self.driver_thread.change_threshold(config.th_h_5, 5)
        self.driver_thread.change_threshold(config.th_h_6, 6)
        self.driver_thread.change_threshold(config.th_h_7, 7)
        self.driver_thread.change_nsamp(config.nsamp_h, 0)
    
    def on_Configuration_apply_m(self):
        config = config_class()
        config = self.config_ui.get_configuration(config)
        self.driver_thread.change_threshold(config.th_m_0, 8)
        self.driver_thread.change_threshold(config.th_m_1, 9)
        self.driver_thread.change_threshold(config.th_m_2, 10)
        self.driver_thread.change_threshold(config.th_m_3, 11)
        self.driver_thread.change_threshold(config.th_m_4, 12)
        self.driver_thread.change_threshold(config.th_m_5, 13)
        self.driver_thread.change_threshold(config.th_m_6, 14)
        self.driver_thread.change_threshold(config.th_m_7, 15)
        self.driver_thread.change_nsamp(config.nsamp_m, 1)

    def on_Configuration_apply_l(self):
        config = config_class()
        config = self.config_ui.get_configuration(config)
        self.driver_thread.change_threshold(config.th_l_0, 16)
        self.driver_thread.change_threshold(config.th_l_1, 17)
        self.driver_thread.change_threshold(config.th_l_2, 18)
        self.driver_thread.change_threshold(config.th_l_3, 19)
        self.driver_thread.change_threshold(config.th_l_4, 20)
        self.driver_thread.change_threshold(config.th_l_5, 21)
        self.driver_thread.change_threshold(config.th_l_6, 22)
        self.driver_thread.change_threshold(config.th_l_7, 23)
        self.driver_thread.change_nsamp(config.nsamp_l, 2)

    def update_configuration_gui(self, th_array, nsamp_array):
        self.config_ui.ui.text_th_h_0.setText("{:1.3e}".format(th_array[0,0]))
        self.config_ui.ui.text_th_h_1.setText("{:1.3e}".format(th_array[1,0]))
        self.config_ui.ui.text_th_h_2.setText("{:1.3e}".format(th_array[2,0]))
        self.config_ui.ui.text_th_h_3.setText("{:1.3e}".format(th_array[3,0]))
        self.config_ui.ui.text_th_h_4.setText("{:1.3e}".format(th_array[4,0]))
        self.config_ui.ui.text_th_h_5.setText("{:1.3e}".format(th_array[5,0]))
        self.config_ui.ui.text_th_h_6.setText("{:1.3e}".format(th_array[6,0]))
        self.config_ui.ui.text_th_h_7.setText("{:1.3e}".format(th_array[7,0]))
        self.config_ui.ui.text_nsamp_h.setText(str(nsamp_array[0,0]))

        self.config_ui.ui.text_th_m_0.setText("{:1.3e}".format(th_array[8,0]))
        self.config_ui.ui.text_th_m_1.setText("{:1.3e}".format(th_array[9,0]))
        self.config_ui.ui.text_th_m_2.setText("{:1.3e}".format(th_array[10,0]))
        self.config_ui.ui.text_th_m_3.setText("{:1.3e}".format(th_array[11,0]))
        self.config_ui.ui.text_th_m_4.setText("{:1.3e}".format(th_array[12,0]))
        self.config_ui.ui.text_th_m_5.setText("{:1.3e}".format(th_array[13,0]))
        self.config_ui.ui.text_th_m_6.setText("{:1.3e}".format(th_array[14,0]))
        self.config_ui.ui.text_th_m_7.setText("{:1.3e}".format(th_array[15,0]))
        self.config_ui.ui.text_nsamp_m.setText(str(nsamp_array[1,0]))

        self.config_ui.ui.text_th_l_0.setText("{:1.3e}".format(th_array[16,0]))
        self.config_ui.ui.text_th_l_1.setText("{:1.3e}".format(th_array[17,0]))
        self.config_ui.ui.text_th_l_2.setText("{:1.3e}".format(th_array[18,0]))
        self.config_ui.ui.text_th_l_3.setText("{:1.3e}".format(th_array[19,0]))
        self.config_ui.ui.text_th_l_4.setText("{:1.3e}".format(th_array[20,0]))
        self.config_ui.ui.text_th_l_5.setText("{:1.3e}".format(th_array[21,0]))
        self.config_ui.ui.text_th_l_6.setText("{:1.3e}".format(th_array[22,0]))
        self.config_ui.ui.text_th_l_7.setText("{:1.3e}".format(th_array[23,0]))
        self.config_ui.ui.text_nsamp_l.setText(str(nsamp_array[2,0]))

    def read_configuration_from_fpga(self):
        th_array = np.zeros([N_CHANNELS*3,1])
        nsamp_array = np.zeros([3,1])
        coeff = 0.0
        nsamp = int(1)
        for ind_i in range(0,N_CHANNELS*3):
            th_array[ind_i,0] = self.driver_thread.read_threshold(data=coeff, addr=ind_i)
        for ind_i in range(0,3):
            nsamp_array[ind_i,0] = self.driver_thread.read_nsamp(data=nsamp, addr=ind_i)
        self.update_configuration_gui(th_array, nsamp_array)




def main():
    app = QtGui.QApplication(sys.argv)

    if '--dev' in sys.argv:
        pico_path = sys.argv[ sys.argv.index('--dev') + 1]
        rc = 1
    else:
        picoSel_win = PicoSelector()
        rc = picoSel_win.exec_()
        pico_path = picoSel_win.get_pico_path()

    if rc:
        window = UserWindow(pico_path)
        app.exec_()
        sys.exit()


if __name__ == '__main__':
    main()
